@@ -1,5 +1,5 @@
 /*
- *            Copyright 2009-2019 The VOTCA Development Team
+ *            Copyright 2009-2020 The VOTCA Development Team
  *                       (http://www.votca.org)
  *
  *      Licensed under the Apache License, Version 2.0 (the "License")
@@ -18,12 +18,14 @@
  */
 
 #pragma once
-#ifndef XTP_AMPLITUDE_INTEGRATION_H
-#define XTP_AMPLITUDE_INTEGRATION_H
+#ifndef VOTCA_XTP_AMPLITUDE_INTEGRATION_H
+#define VOTCA_XTP_AMPLITUDE_INTEGRATION_H
+
+// Local VOTCA includes
+#include "aobasis.h"
+#include "eigen.h"
+#include "regular_grid.h"
 
-#include <votca/xtp/aobasis.h>
-#include <votca/xtp/eigen.h>
-#include <votca/xtp/regular_grid.h>
 namespace votca {
 namespace xtp {
 
@@ -42,4 +44,4 @@ class AmplitudeIntegration {
 
 }  // namespace xtp
 }  // namespace votca
-#endif  // XTP_NUMERICAL_INTEGRATION_H
+#endif  // VOTCA_XTP_AMPLITUDE_INTEGRATION_H
