@@ -1,5 +1,5 @@
 /*
- *            Copyright 2009-2019 The VOTCA Development Team
+ *            Copyright 2009-2020 The VOTCA Development Team
  *                       (http://www.votca.org)
  *
  *      Licensed under the Apache License, Version 2.0 (the "License")
@@ -21,8 +21,11 @@
 #ifndef VOTCA_XTP_DIIS_H
 #define VOTCA_XTP_DIIS_H
 
+// Standard includes
 #include <vector>
-#include <votca/xtp/eigen.h>
+
+// Local VOTCA includes
+#include "eigen.h"
 
 namespace votca {
 namespace xtp {
