@@ -1,5 +1,5 @@
 /*
- *            Copyright 2009-2019 The VOTCA Development Team
+ *            Copyright 2009-2020 The VOTCA Development Team
  *                       (http://www.votca.org)
  *
  *      Licensed under the Apache License, Version 2.0 (the "License")
@@ -21,8 +21,11 @@
 #ifndef VOTCA_XTP_FILTERFACTORY_H
 #define VOTCA_XTP_FILTERFACTORY_H
 
+// VOTCA includes
 #include <votca/tools/objectfactory.h>
-#include <votca/xtp/statefilter_base.h>
+
+// Local VOTCA includes
+#include "statefilter_base.h"
 
 namespace votca {
 namespace xtp {
@@ -45,4 +48,4 @@ inline FilterFactory &Filter() {
 }  // namespace xtp
 }  // namespace votca
 
-#endif  // VOTCA_XTP_QMPACKAGEFACTORY_H
+#endif  // VOTCA_XTP_FILTERFACTORY_H
