@@ -1,5 +1,5 @@
 /*
- *            Copyright 2009-2019 The VOTCA Development Team
+ *            Copyright 2009-2020 The VOTCA Development Team
  *                       (http://www.votca.org)
  *
  *      Licensed under the Apache License, Version 2.0 (the "License")
@@ -21,9 +21,14 @@
 #ifndef VOTCA_XTP_JOBCALCULATORFACTORY_H
 #define VOTCA_XTP_JOBCALCULATORFACTORY_H
 
+// Standard includes
 #include <map>
+
+// VOTCA includes
 #include <votca/tools/objectfactory.h>
-#include <votca/xtp/jobcalculator.h>
+
+// Local VOTCA includes
+#include "jobcalculator.h"
 
 namespace votca {
 namespace xtp {
@@ -36,12 +41,6 @@ class JobCalculatorfactory
  public:
   static void RegisterAll(void);
 
-  /**
-     Create an instance of the object identified by key.
-  *  Overwritten to load calculator defaults
-  */
-  JobCalculator *Create(const std::string &key);
-
   friend JobCalculatorfactory &JobCalculators();
 };
 
@@ -50,16 +49,6 @@ inline JobCalculatorfactory &JobCalculators() {
   return _instance;
 }
 
-inline JobCalculator *JobCalculatorfactory::Create(const std::string &key) {
-  assoc_map::const_iterator it(getObjects().find(key));
-  if (it != getObjects().end()) {
-    JobCalculator *calc = (it->second)();
-    return calc;
-  } else {
-    throw std::runtime_error("factory key " + key + " not found.");
-  }
-}
-
 }  // namespace xtp
 }  // namespace votca
 
