@@ -1,5 +1,5 @@
 /*
- *            Copyright 2009-2019 The VOTCA Development Team
+ *            Copyright 2009-2020 The VOTCA Development Team
  *                       (http://www.votca.org)
  *
  *      Licensed under the Apache License, Version 2.0 (the "License")
@@ -21,10 +21,10 @@
 #ifndef VOTCA_XTP_STATEAPPLICATION_H
 #define VOTCA_XTP_STATEAPPLICATION_H
 
-#include <votca/xtp/qmcalculator.h>
-#include <votca/xtp/topology.h>
-
-#include <votca/xtp/xtpapplication.h>
+// Local VOTCA includes
+#include "qmcalculator.h"
+#include "topology.h"
+#include "xtpapplication.h"
 
 namespace votca {
 namespace xtp {
@@ -42,7 +42,7 @@ class StateApplication : public XtpApplication {
   void BeginEvaluate(Index nThreads);
   bool EvaluateFrame(Topology& top);
 
-  void SetCalculator(QMCalculator* calculator);
+  void SetCalculator(std::unique_ptr<QMCalculator>&& calculator);
 
  protected:
   std::unique_ptr<QMCalculator> _calculator;
@@ -51,4 +51,4 @@ class StateApplication : public XtpApplication {
 }  // namespace xtp
 }  // namespace votca
 
-#endif  // VOTCA_XTP_SQLAPPLICATION_H
+#endif  // VOTCA_XTP_STATEAPPLICATION_H
