@@ -1,5 +1,5 @@
 /*
- *            Copyright 2009-2019 The VOTCA Development Team
+ *            Copyright 2009-2020 The VOTCA Development Team
  *                       (http://www.votca.org)
  *
  *      Licensed under the Apache License, Version 2.0 (the "License")
@@ -17,17 +17,22 @@
  *
  */
 
-#include "iqm.h"
-#include "votca/xtp/segmentmapper.h"
+// Third party includes
 #include <boost/algorithm/string/split.hpp>
 #include <boost/filesystem.hpp>
 #include <boost/format.hpp>
 
+// VOTCA includes
 #include <votca/tools/constants.h>
 
-#include <votca/xtp/atom.h>
-#include <votca/xtp/logger.h>
-#include <votca/xtp/qmpackagefactory.h>
+// Local VOTCA includes
+#include "votca/xtp/atom.h"
+#include "votca/xtp/logger.h"
+#include "votca/xtp/qmpackagefactory.h"
+#include "votca/xtp/segmentmapper.h"
+
+// Local private VOTCA includes
+#include "iqm.h"
 
 using boost::format;
 using namespace boost::filesystem;
@@ -35,23 +40,12 @@ using namespace boost::filesystem;
 namespace votca {
 namespace xtp {
 
-void IQM::Initialize(tools::Property& options) {
-  ParseOptionsXML(options);
+void IQM::ParseSpecificOptions(const tools::Property& options) {
 
-  // register all QM packages (Gaussian, turbomole, etc))
   QMPackageFactory::RegisterAll();
-  return;
-}
-
-void IQM::ParseOptionsXML(tools::Property& opt) {
-
-  ParseCommonOptions(opt);
-  // parsing general ibse options
-  std::string key = "options." + Identify();
-  // _energy_difference = opt.get( key + ".degeneracy" ).as< double > ();
 
   // job tasks
-  std::string tasks_string = opt.get(key + ".tasks").as<std::string>();
+  std::string tasks_string = options.get(".tasks").as<std::string>();
   if (tasks_string.find("input") != std::string::npos) {
     _do_dft_input = true;
   }
@@ -72,7 +66,7 @@ void IQM::ParseOptionsXML(tools::Property& opt) {
   }
 
   // storage options
-  std::string store_string = opt.get(key + ".store").as<std::string>();
+  std::string store_string = options.get(".store").as<std::string>();
   if (store_string.find("dft") != std::string::npos) {
     _store_dft = true;
   }
@@ -80,14 +74,17 @@ void IQM::ParseOptionsXML(tools::Property& opt) {
     _store_gw = true;
   }
 
-  if (_do_dft_input || _do_dft_run || _do_dft_parse) {
-    std::string package_xml = opt.get(key + ".dftpackage").as<std::string>();
-    _dftpackage_options.LoadFromXML(package_xml);
-  }
+  _dftpackage_options = this->UpdateDFTOptions(options);
+  _gwbse_options = this->UpdateGWBSEOptions(options);
+
+  _dftcoupling_options = options.get(".dftcoupling_options");
+  tools::Property& prop_bsecoupling =
+      _bsecoupling_options.add("bsecoupling", "");
+  prop_bsecoupling = options.get("bsecoupling");
 
   // read linker groups
-  std::string linker = opt.ifExistsReturnElseReturnDefault<std::string>(
-      key + ".linker_names", "");
+  std::string linker =
+      options.ifExistsReturnElseReturnDefault<std::string>(".linker_names", "");
   tools::Tokenizer toker(linker, ", \t\n");
   std::vector<std::string> linkers = toker.ToVector();
   for (const std::string& link : linkers) {
@@ -100,45 +97,29 @@ void IQM::ParseOptionsXML(tools::Property& opt) {
     _linkers[link_split[0]] = QMState(link_split[1]);
   }
 
-  if (_do_dftcoupling) {
-    _dftcoupling_options = opt.get(key + ".dftcoupling_options");
-  }
-
-  if (_do_gwbse) {
-    std::string _gwbse_xml = opt.get(key + ".gwbse_options").as<std::string>();
-
-    _gwbse_options.LoadFromXML(_gwbse_xml);
-  }
-  if (_do_bsecoupling) {
-    std::string _coupling_xml =
-        opt.get(key + ".bsecoupling_options").as<std::string>();
-    _bsecoupling_options.LoadFromXML(_coupling_xml);
-  }
-
   // options for parsing data into state file
-  std::string key_read = "options." + Identify() + ".readjobfile";
-  if (opt.exists(key_read + ".singlet")) {
+  std::string key_read = ".readjobfile";
+  if (options.exists(key_read + ".singlet")) {
     std::string parse_string_s =
-        opt.get(key_read + ".singlet").as<std::string>();
+        options.get(key_read + ".singlet").as<std::string>();
     _singlet_levels = FillParseMaps(parse_string_s);
   }
-  if (opt.exists(key_read + ".triplet")) {
+  if (options.exists(key_read + ".triplet")) {
     std::string parse_string_t =
-        opt.get(key_read + ".triplet").as<std::string>();
+        options.get(key_read + ".triplet").as<std::string>();
     _triplet_levels = FillParseMaps(parse_string_t);
   }
 
-  if (opt.exists(key_read + ".hole")) {
-    std::string parse_string_h = opt.get(key_read + ".hole").as<std::string>();
+  if (options.exists(key_read + ".hole")) {
+    std::string parse_string_h =
+        options.get(key_read + ".hole").as<std::string>();
     _hole_levels = FillParseMaps(parse_string_h);
   }
-  if (opt.exists(key_read + ".electron")) {
+  if (options.exists(key_read + ".electron")) {
     std::string parse_string_e =
-        opt.get(key_read + ".electron").as<std::string>();
+        options.get(key_read + ".electron").as<std::string>();
     _electron_levels = FillParseMaps(parse_string_e);
   }
-
-  return;
 }
 
 std::map<std::string, QMState> IQM::FillParseMaps(
@@ -175,7 +156,6 @@ void IQM::addLinkers(std::vector<const Segment*>& segments,
       segments.push_back(segment);
     }
   }
-  return;
 }
 
 bool IQM::isLinker(const std::string& name) {
@@ -313,7 +293,6 @@ Job::JobResult IQM::EvalJob(const Topology& top, Job& job, QMThread& opThread) {
     std::string qmpackage_work_dir =
         (arg_path / iqm_work_dir / package_append / frame_dir / pair_dir)
             .generic_string();
-    ;
 
     Logger dft_logger(Log::current_level);
     dft_logger.setMultithreading(false);
@@ -323,8 +302,8 @@ Job::JobResult IQM::EvalJob(const Topology& top, Job& job, QMThread& opThread) {
     dft_logger.setPreface(Log::debug, (format("\nDFT DBG ...")).str());
     std::string dftname = "package.name";
     std::string package = _dftpackage_options.get(dftname).as<std::string>();
-    std::unique_ptr<QMPackage> qmpackage =
-        std::unique_ptr<QMPackage>(QMPackages().Create(package));
+    std::unique_ptr<QMPackage> qmpackage = std::unique_ptr<QMPackage>(
+        QMPackageFactory::QMPackages().Create(package));
     qmpackage->setLog(&dft_logger);
     qmpackage->setRunDir(qmpackage_work_dir);
     qmpackage->Initialize(_dftpackage_options);
@@ -412,6 +391,7 @@ Job::JobResult IQM::EvalJob(const Topology& top, Job& job, QMThread& opThread) {
       bool _run_dft_status = qmpackage->Run();
       if (!_run_dft_status) {
         SetJobToFailed(jres, pLog, qmpackage->getPackageName() + " run failed");
+        WriteLoggerToFile(work_dir + "/dft.log", dft_logger);
         return jres;
       }
     }
