@@ -1,5 +1,5 @@
 /*
- *            Copyright 2009-2019 The VOTCA Development Team
+ *            Copyright 2009-2020 The VOTCA Development Team
  *                       (http://www.votca.org)
  *
  *      Licensed under the Apache License, Version 2.0 (the "License")
@@ -17,17 +17,19 @@
  *
  */
 
-#include "votca/xtp/orbitals.h"
-#include "votca/xtp/aomatrix.h"
-#include "votca/xtp/aomatrix3d.h"
-#include "votca/xtp/qmstate.h"
+// Standard includes
+#include <cstdio>
 #include <fstream>
 #include <iomanip>
-#include <iostream>
 #include <numeric>
-#include <stdio.h>
-#include <votca/xtp/vc2index.h>
-#include <votca/xtp/version.h>
+
+// Local VOTCA includes
+#include "votca/xtp/aomatrix.h"
+#include "votca/xtp/orbitals.h"
+#include "votca/xtp/orbreorder.h"
+#include "votca/xtp/qmstate.h"
+#include "votca/xtp/vc2index.h"
+#include "votca/xtp/version.h"
 
 namespace votca {
 namespace xtp {
@@ -503,7 +505,8 @@ void Orbitals::WriteToCpt(CheckpointFile f) const {
 }
 
 void Orbitals::WriteToCpt(CheckpointWriter w) const {
-  w(XtpVersionStr(), "Version");
+  w(XtpVersionStr(), "XTPVersion");
+  w(orbitals_version(), "version");
   w(_basis_set_size, "basis_set_size");
   w(_occupied_levels, "occupied_levels");
   w(_number_alpha_electrons, "number_alpha_electrons");
@@ -531,6 +534,7 @@ void Orbitals::WriteToCpt(CheckpointWriter w) const {
   w(_useTDA, "useTDA");
   w(_ECP, "ECP");
 
+  w(_rpa_inputenergies, "RPA_inputenergies");
   w(_QPpert_energies, "QPpert_energies");
 
   w(_QPdiag, "QPdiag");
@@ -540,6 +544,12 @@ void Orbitals::WriteToCpt(CheckpointWriter w) const {
   w(_transition_dipoles, "transition_dipoles");
 
   w(_BSE_triplet, "BSE_triplet");
+
+  w(_use_Hqp_offdiag, "use_Hqp_offdiag");
+
+  w(_BSE_singlet_energies_dynamic, "BSE_singlet_dynamic");
+
+  w(_BSE_triplet_energies_dynamic, "BSE_triplet_dynamic");
 }
 
 void Orbitals::ReadFromCpt(const std::string& filename) {
@@ -555,9 +565,8 @@ void Orbitals::ReadFromCpt(CheckpointReader r) {
   r(_basis_set_size, "basis_set_size");
   r(_occupied_levels, "occupied_levels");
   r(_number_alpha_electrons, "number_alpha_electrons");
-
-  r(_mos, "mos");
-
+  int version;
+  r(version, "version");
   // Read qmatoms
   CheckpointReader molgroup = r.openChild("qmmolecule");
   _atoms.ReadFromCpt(molgroup);
@@ -568,6 +577,27 @@ void Orbitals::ReadFromCpt(CheckpointReader r) {
   r(_dftbasis, "dftbasis");
   r(_auxbasis, "auxbasis");
 
+  r(version, "version");
+  r(_mos, "mos");
+  if (version < 3) {
+    // clang-format off
+    std::array<Index, 49> votcaOrder_old = {
+        0,                             // s
+        0, -1, 1,                      // p
+        0, -1, 1, -2, 2,               // d
+        0, -1, 1, -2, 2, -3, 3,        // f
+        0, -1, 1, -2, 2, -3, 3, -4, 4,  // g
+        0, -1, 1, -2, 2, -3, 3, -4, 4,-5,5,  // h
+        0, -1, 1, -2, 2, -3, 3, -4, 4,-5,5,-6,6  // i
+    };
+    // clang-format on
+
+    std::array<Index, 49> multiplier;
+    multiplier.fill(1);
+    OrbReorder ord(votcaOrder_old, multiplier);
+    ord.reorderOrbitals(_mos.eigenvectors(), this->SetupDftBasis());
+  }
+
   r(_rpamin, "rpamin");
   r(_rpamax, "rpamax");
   r(_qpmin, "qpmin");
@@ -584,6 +614,7 @@ void Orbitals::ReadFromCpt(CheckpointReader r) {
   r(_useTDA, "useTDA");
   r(_ECP, "ECP");
 
+  r(_rpa_inputenergies, "RPA_inputenergies");
   r(_QPpert_energies, "QPpert_energies");
   r(_QPdiag, "QPdiag");
 
@@ -592,6 +623,14 @@ void Orbitals::ReadFromCpt(CheckpointReader r) {
   r(_transition_dipoles, "transition_dipoles");
 
   r(_BSE_triplet, "BSE_triplet");
+
+  r(_use_Hqp_offdiag, "use_Hqp_offdiag");
+
+  if (version > 1) {
+    r(_BSE_singlet_energies_dynamic, "BSE_singlet_dynamic");
+
+    r(_BSE_triplet_energies_dynamic, "BSE_triplet_dynamic");
+  }
 }
 }  // namespace xtp
 }  // namespace votca
