@@ -17,27 +17,40 @@
  *
  */
 
-#include "orca.h"
+// Standard includes
+#include <cstdio>
+#include <iomanip>
+
+// Third party includes
 #include <boost/algorithm/string.hpp>
 #include <boost/filesystem.hpp>
 #include <boost/format.hpp>
-#include <iomanip>
-#include <stdio.h>
+
+// VOTCA includes
 #include <votca/tools/elements.h>
 #include <votca/tools/getline.h>
-#include <votca/xtp/ecpaobasis.h>
-#include <votca/xtp/orbitals.h>
+
+// Local VOTCA includes
+#include "votca/tools/globals.h"
+#include "votca/xtp/basisset.h"
+#include "votca/xtp/ecpaobasis.h"
+#include "votca/xtp/orbitals.h"
+
+// Local private VOTCA includes
+#include "orca.h"
 
 namespace votca {
 namespace xtp {
 using namespace std;
 
-void Orca::Initialize(tools::Property& options) {
+void Orca::Initialize(const tools::Property& options) {
 
   // good luck
 
   // Orca file names
-  std::string fileName = "system";
+  const std::string& fileName =
+      options.ifExistsReturnElseReturnDefault<std::string>("job_name",
+                                                           "system");
 
   _input_file_name = fileName + ".inp";
   _log_file_name = fileName + ".log";
@@ -45,20 +58,6 @@ void Orca::Initialize(tools::Property& options) {
   _mo_file_name = fileName + ".gbw";
 
   ParseCommonOptions(options);
-
-  // check if the optimize keyword is present, if yes, read updated coords
-  std::string::size_type iop_pos =
-      _options.find(" Opt"); /*optimization word in orca*/
-  if (iop_pos != std::string::npos) {
-    _is_optimization = true;
-  }
-
-  if (_write_guess) {
-    iop_pos = _options.find("Guess MORead");
-    if (iop_pos != std::string::npos) {
-      _options = _options + "\n Guess MORead ";
-    }
-  }
 }
 
 /* Custom basis sets are written on a per-element basis to
@@ -83,17 +82,14 @@ void Orca::WriteBasisset(const QMMolecule& qmatoms, std::string& bs_name,
     const Element& element = bs.getElement(element_name);
     el_file << elementInfo.getEleFull(element_name) << endl;
     for (const Shell& shell : element) {
-      for (const char& subtype : shell.getType()) {
-        el_file << subtype << " " << shell.getSize() << endl;
-        Index sh_idx = 0;
-        for (const GaussianPrimitive& gaussian : shell) {
-          sh_idx++;
-          el_file << " " << sh_idx << " " << indent(gaussian.decay());
-          el_file << " "
-                  << indent(gaussian.Contractions()[FindLmax(
-                         std::string(1, subtype))]);
-          el_file << endl;
-        }
+      el_file << xtp::EnumToString(shell.getL()) << " " << shell.getSize()
+              << endl;
+      Index sh_idx = 0;
+      for (const GaussianPrimitive& gaussian : shell) {
+        sh_idx++;
+        el_file << " " << sh_idx << " " << indent(gaussian.decay());
+        el_file << " " << indent(gaussian.contraction());
+        el_file << endl;
       }
     }
   }
@@ -130,10 +126,10 @@ void Orca::WriteECP(std::ofstream& inp_file, const QMMolecule& qmatoms) {
   std::vector<std::string> UniqueElements = qmatoms.FindUniqueElements();
 
   ECPBasisSet ecp;
-  ecp.Load(_ecp_name);
+  ecp.Load(_settings.get("ecp"));
 
   XTP_LOG(Log::error, *_pLog)
-      << "Loaded Pseudopotentials " << _ecp_name << flush;
+      << "Loaded Pseudopotentials " << _settings.get("ecp") << flush;
 
   for (const std::string& element_name : UniqueElements) {
     try {
@@ -151,14 +147,15 @@ void Orca::WriteECP(std::ofstream& inp_file, const QMMolecule& qmatoms) {
     inp_file << "N_core"
              << " " << element.getNcore() << endl;
     inp_file << "lmax"
-             << " " << getLName(element.getLmax()) << endl;
+             << " " << EnumToString(element.getLmax()) << endl;
     // For Orca the order doesn't matter but let's write it in ascending order
     // write remaining shells in ascending order s,p,d...
-    for (Index i = 0; i <= element.getLmax(); i++) {
+    for (Index i = 0; i <= Index(element.getLmax()); i++) {
       for (const ECPShell& shell : element) {
-        if (shell.getL() == i) {
+        if (Index(shell.getL()) == i) {
           // shell type, number primitives, scale factor
-          inp_file << shell.getType() << " " << shell.getSize() << endl;
+          inp_file << xtp::EnumToString(shell.getL()) << " " << shell.getSize()
+                   << endl;
           Index sh_idx = 0;
           for (const ECPGaussianPrimitive& gaussian : shell) {
             sh_idx++;
@@ -176,10 +173,7 @@ void Orca::WriteECP(std::ofstream& inp_file, const QMMolecule& qmatoms) {
 }
 
 void Orca::WriteChargeOption() {
-  std::string::size_type iop_pos = _options.find("pointcharges");
-  if (iop_pos == std::string::npos) {
-    _options = _options + "\n %pointcharges \"background.crg\"";
-  }
+  this->_settings.add("orca.pointcharges", "\"background.crg\"");
 }
 
 /* For QM/MM the molecules in the MM environment are represented by
@@ -243,42 +237,51 @@ bool Orca::WriteInputFile(const Orbitals& orbitals) {
   // put coordinates
   WriteCoordinates(inp_file, qmatoms);
   // add parallelization info
-  inp_file << "%pal\n "
+  inp_file << "%pal\n"
            << "nprocs " << threads << "\nend"
            << "\n"
            << endl;
   // basis set info
-  if (_write_basis_set) {
-    std::string el_file_name = _run_dir + "/" + "system.bas";
-    WriteBasisset(qmatoms, _basisset_name, el_file_name);
-    inp_file << "%basis\n " << endl;
-    inp_file << "GTOName"
+  std::string el_file_name = _run_dir + "/" + "system.bas";
+  WriteBasisset(qmatoms, _basisset_name, el_file_name);
+  inp_file << "%basis\n";
+  inp_file << "GTOName"
+           << " "
+           << "="
+           << "\"system.bas\";" << endl;
+  if (_settings.has_key("auxbasisset")) {
+    std::string aux_file_name = _run_dir + "/" + "system.aux";
+    std::string auxbasisset_name = _settings.get("auxbasisset");
+    WriteBasisset(qmatoms, auxbasisset_name, aux_file_name);
+    inp_file << "GTOAuxName"
              << " "
              << "="
-             << "\"system.bas\";" << endl;
-    if (_write_auxbasis_set) {
-      std::string aux_file_name = _run_dir + "/" + "system.aux";
-      WriteBasisset(qmatoms, _auxbasisset_name, aux_file_name);
-      inp_file << "GTOAuxName"
-               << " "
-               << "="
-               << "\"system.aux\";" << endl;
-    }
-  }  // write_basis set
+             << "\"system.aux\";" << endl;
+  }  // write_auxbasis set
 
   // ECPs
-  if (_write_pseudopotentials) {
+  if (_settings.has_key("ecp")) {
     WriteECP(inp_file, qmatoms);
   }
   inp_file << "end\n "
            << "\n"
            << endl;  // This end is for the basis set block
-  if (_write_charges) {
+  if (_settings.get<bool>("write_charges")) {
     WriteBackgroundCharges();
   }
 
-  inp_file << _options << "\n";
-  inp_file << endl;
+  // Write Orca section specified by the user
+  for (const auto& prop : this->_settings.property("orca")) {
+    const std::string& prop_name = prop.name();
+    if (prop_name == "pointcharges") {
+      _options += this->CreateInputSection("orca.pointcharges");
+    } else if (prop_name != "method") {
+      _options += this->CreateInputSection("orca." + prop_name);
+    }
+  }
+  // Write main DFT method
+  _options += this->WriteMethod();
+  inp_file << _options;
   inp_file.close();
   // and now generate a shell script to run both jobs, if neccessary
 
@@ -297,17 +300,17 @@ bool Orca::WriteShellScript() {
   shell_file << "#!/bin/bash" << endl;
   shell_file << "mkdir -p " << _scratch_dir << endl;
 
-  if (_write_guess) {
+  if (_settings.get<bool>("read_guess")) {
     if (!(boost::filesystem::exists(_run_dir + "/molA.gbw") &&
           boost::filesystem::exists(_run_dir + "/molB.gbw"))) {
       throw runtime_error(
           "Using guess relies on a molA.gbw and a molB.gbw file being in the "
           "directory.");
     }
-    shell_file << _executable
+    shell_file << _settings.get("executable")
                << "_mergefrag molA.gbw molB.gbw dimer.gbw > merge.log" << endl;
   }
-  shell_file << _executable << " " << _input_file_name << " > "
+  shell_file << _settings.get("executable") << " " << _input_file_name << " > "
              << _log_file_name << endl;  //" 2> run.error" << endl;
   shell_file.close();
   return true;
@@ -349,7 +352,7 @@ bool Orca::Run() {
  */
 void Orca::CleanUp() {
 
-  if (_write_guess) {
+  if (_settings.get<bool>("read_guess")) {
     remove((_run_dir + "/" + "molA.gbw").c_str());
     remove((_run_dir + "/" + "molB.gbw").c_str());
     remove((_run_dir + "/" + "dimer.gbw").c_str());
@@ -461,7 +464,7 @@ Eigen::Matrix3d Orca::GetPolarizability() const {
       if (line.find("The raw cartesian tensor (atomic units)") ==
           std::string::npos) {
         throw std::runtime_error(
-            "Could not find cartesian polarisation tensor");
+            "Could not find cartesian polarization tensor");
       }
 
       for (Index i = 0; i < 3; i++) {
@@ -469,7 +472,7 @@ Eigen::Matrix3d Orca::GetPolarizability() const {
         tools::Tokenizer tok2(line, " ");
         std::vector<std::string> values = tok2.ToVector();
         if (values.size() != 3) {
-          throw std::runtime_error("Polarisation line " + line +
+          throw std::runtime_error("polarization line " + line +
                                    " cannot be parsed");
         }
         Eigen::Vector3d row;
@@ -481,7 +484,7 @@ Eigen::Matrix3d Orca::GetPolarizability() const {
     }
   }
   if (!has_pol) {
-    throw std::runtime_error("Could not find polarisation in logfile");
+    throw std::runtime_error("Could not find polarization in logfile");
   }
   return pol;
 }
@@ -490,8 +493,8 @@ bool Orca::ParseLogFile(Orbitals& orbitals) {
   bool found_success = false;
   orbitals.setQMpackage(getPackageName());
   orbitals.setDFTbasisName(_basisset_name);
-  if (_write_pseudopotentials) {
-    orbitals.setECPName(_ecp_name);
+  if (_settings.has_key("ecp")) {
+    orbitals.setECPName(_settings.get("ecp"));
   }
 
   XTP_LOG(Log::error, *_pLog) << "Parsing " << _log_file_name << flush;
@@ -789,22 +792,6 @@ bool Orca::ParseMOsFile(Orbitals& orbitals) {
   return true;
 }
 
-std::string Orca::getLName(Index lnum) {
-  if (lnum == 0) {
-    return "S";
-  } else if (lnum == 1) {
-    return "P";
-  } else if (lnum == 2) {
-    return "D";
-  } else if (lnum == 3) {
-    return "F";
-  } else {
-    throw runtime_error(
-        "Orca::getLName functions higher than F not implemented");
-  }
-  return "0";
-}
-
 std::string Orca::indent(const double& number) {
   std::stringstream ssnumber;
   if (number >= 0) {
@@ -818,5 +805,83 @@ std::string Orca::indent(const double& number) {
   return snumber;
 }
 
+std::string Orca::CreateInputSection(const std::string& key) const {
+  std::stringstream stream;
+  std::string section = key.substr(key.find(".") + 1);
+  stream << "%" << section;
+  if (KeywordIsSingleLine(key)) {
+    stream << " " << this->_settings.get(key) << "\n";
+  } else {
+    stream << "\n"
+           << this->_settings.get(key) << "\n"
+           << "end\n";
+  }
+
+  return stream.str();
+}
+
+bool Orca::KeywordIsSingleLine(const std::string& key) const {
+  tools::Tokenizer values(this->_settings.get(key), " ");
+  std::vector<std::string> words;
+  values.ToVector(words);
+  return ((words.size() <= 1) ? true : false);
+}
+
+std::string Orca::WriteMethod() const {
+  std::stringstream stream;
+  std::string opt = (_settings.get<bool>("optimize")) ? "Opt" : "";
+  const tools::Property& orca = _settings.property("orca");
+  std::string user_method =
+      (orca.exists("method")) ? orca.get("method").as<std::string>() : "";
+  std::string convergence = "";
+  if (!orca.exists("scf")) {
+    convergence =
+        this->_convergence_map.at(_settings.get("convergence_tightness"));
+  }
+  stream << "! DFT " << this->GetOrcaFunctionalName() << " " << convergence
+         << " " << opt
+         << " "
+         // additional properties provided by the user
+         << user_method << "\n";
+  return stream.str();
+}
+
+std::string Orca::GetOrcaFunctionalName() const {
+
+  if (!tools::VotcaShareSet()) {
+    return _settings.get("functional");
+  } else {
+    tools::Property all_functionals;
+
+    auto xml_file =
+        tools::GetVotcaShare() + "/xtp/data/orca_functional_names.xml";
+
+    all_functionals.LoadFromXML(xml_file);
+
+    const tools::Property& functional_names =
+        all_functionals.get("functionals");
+
+    std::string input_name = _settings.get("functional");
+    // Some functionals have a composed named separated by a space
+    // In the case just look for the first part
+    std::size_t plus = input_name.find(' ');
+    if (plus != std::string::npos) {
+      input_name = input_name.substr(0, plus);
+    }
+
+    if (functional_names.exists(input_name)) {
+      return functional_names.get(input_name).as<std::string>();
+    } else {
+      std::ostringstream oss;
+      oss << "The libxc functional \"" << input_name << "\"\n"
+          << "doesn't seem to have a corresponding name in Orca.\n"
+          << "Check the "
+          << "\"${VOTCASHARE}/xtp/data/orca_functional_names.xml\""
+          << "file for the whole list of known libxc/orca functionals";
+      throw runtime_error(oss.str());
+    }
+  }
+}
+
 }  // namespace xtp
 }  // namespace votca
